@@ -46,6 +46,11 @@ void Class::setAttribute(instance_ptr self, int64_t ix, instance_ptr elt) const
     m_heldClass->setAttribute(l.data, ix, elt);
 }
 
+void Class::delAttribute(instance_ptr self, int64_t ix) const {
+    layout& l = *instanceToLayout(self);
+    m_heldClass->delAttribute(l.data, ix);
+}
+
 bool Class::checkInitializationFlag(instance_ptr self, int64_t ix) const {
     layout& l = *instanceToLayout(self);
     return m_heldClass->checkInitializationFlag(l.data, ix);
@@ -201,13 +206,13 @@ void Class::constructor(instance_ptr self, bool allowEmpty) {
         throw std::runtime_error(m_name + " is not default-constructible");
     }
 
-    initializeInstance(self, (layout*)malloc(sizeof(layout) + m_heldClass->bytecount()), 0);
+    initializeInstance(self, (layout*)tp_malloc(sizeof(layout) + m_heldClass->bytecount()), 0);
 
     layout& l = *instanceToLayout(self);
     l.refcount = 1;
     l.vtable = m_heldClass->getVTable();
 
-    m_heldClass->constructor(l.data);
+    m_heldClass->constructor(l.data, allowEmpty);
 }
 
 int64_t Class::refcount(instance_ptr self) {
@@ -219,7 +224,7 @@ void Class::destroy(instance_ptr self) {
 
     if (l.refcount.fetch_sub(1) == 1) {
         l.vtable->mType->destroy(l.data);
-        free(instanceToLayout(self));
+        tp_free(instanceToLayout(self));
     }
 }
 
@@ -239,6 +244,6 @@ void Class::assign(instance_ptr self, instance_ptr other) {
 
     if (old->refcount.fetch_sub(1) == 1) {
         old->vtable->mType->destroy(old->data);
-        free(old);
+        tp_free(old);
     }
 }
