@@ -126,8 +126,10 @@ public:
             const std::map<std::string, Function*>& memberFunctions,
             const std::map<std::string, Function*>& staticFunctions,
             const std::map<std::string, Function*>& propertyFunctions,
-            const std::map<std::string, PyObject*>& classMembers
-            )
+            const std::map<std::string, PyObject*>& classMembers,
+            const std::map<std::string, Function*>& classMethods,
+            bool isNew
+        )
     {
         std::vector<HeldClass*> heldClassBases;
 
@@ -145,7 +147,9 @@ public:
                 memberFunctions,
                 staticFunctions,
                 propertyFunctions,
-                classMembers
+                classMembers,
+                classMethods,
+                isNew
             )
         );
     }
@@ -451,6 +455,10 @@ public:
         return m_heldClass->getStaticFunctions();
     }
 
+    const std::map<std::string, Function*>& getClassMethods() const {
+        return m_heldClass->getClassMethods();
+    }
+
     const std::map<std::string, PyObject*>& getClassMembers() const {
         return m_heldClass->getClassMembers();
     }
@@ -475,6 +483,10 @@ public:
         return m_heldClass->getOwnClassMembers();
     }
 
+    const std::map<std::string, Function*>& getOwnClassMethods() const {
+        return m_heldClass->getOwnClassMethods();
+    }
+
     const std::map<std::string, Function*>& getOwnPropertyFunctions() const {
         return m_heldClass->getOwnPropertyFunctions();
     }
