@@ -338,7 +338,12 @@ public:
           const std::map<std::string, Function*>& memberFunctions,
           const std::map<std::string, Function*>& staticFunctions,
           const std::map<std::string, Function*>& propertyFunctions,
-          const std::map<std::string, PyObject*>& classMembers
+          const std::map<std::string, PyObject*>& classMembers,
+          const std::map<std::string, Function*>& classMethods,
+          // set to True if this is the first time the class is being created
+          // and we need to make copies of all the function objects so that
+          // they know who their methods are.
+          bool isNew
           ) :
             Type(catHeldClass),
             m_vtable(new VTable(this)),
@@ -350,6 +355,7 @@ public:
             m_own_staticFunctions(staticFunctions),
             m_own_propertyFunctions(propertyFunctions),
             m_own_classMembers(classMembers),
+            m_own_classMethods(classMethods),
             m_hasComparisonOperators(false),
             m_hasGetAttributeMagicMethod(false),
             m_hasGetAttrMagicMethod(false),
@@ -358,6 +364,20 @@ public:
             m_refToType(nullptr)
     {
         m_name = inName;
+
+        if (isNew) {
+            for (auto& nameAndF: m_own_memberFunctions) {
+                nameAndF.second = nameAndF.second->withMethodOf(this);
+            }
+
+            for (auto& nameAndF: m_own_classMethods) {
+                nameAndF.second = nameAndF.second->withMethodOf(this);
+            }
+
+            for (auto& nameAndF: m_own_staticFunctions) {
+                nameAndF.second = nameAndF.second->withMethodOf(this);
+            }
+        }
     }
 
     ShaHash _computeIdentityHash(MutuallyRecursiveTypeGroup* groupHead = nullptr) {
@@ -400,6 +420,12 @@ public:
             res += MutuallyRecursiveTypeGroup::pyObjectShaHash(nameAndFun.second, groupHead);
         }
 
+        res += ShaHash(6);
+        for (auto nameAndFun: m_own_classMethods) {
+            res += ShaHash(nameAndFun.first);
+            res += nameAndFun.second->identityHash(groupHead);
+        }
+
         return res;
     }
 
@@ -443,6 +469,11 @@ public:
             visitor(t);
             assert(t == o.second);
         }
+        for (auto& o: m_own_classMethods) {
+            Type* t = o.second;
+            visitor(t);
+            assert(t == o.second);
+        }
         for (auto& o: m_own_propertyFunctions) {
             Type* t = o.second;
             visitor(t);
@@ -487,7 +518,9 @@ public:
         const std::map<std::string, Function*>& memberFunctions,
         const std::map<std::string, Function*>& staticFunctions,
         const std::map<std::string, Function*>& propertyFunctions,
-        const std::map<std::string, PyObject*>& classMembers
+        const std::map<std::string, PyObject*>& classMembers,
+        const std::map<std::string, Function*>& classMethods,
+        bool isNew
     );
 
     // this gets called by Class. These types are always produced in pairs.
@@ -514,7 +547,9 @@ public:
             m_own_memberFunctions,
             m_own_staticFunctions,
             m_own_propertyFunctions,
-            m_own_classMembers
+            m_own_classMembers,
+            m_own_classMethods,
+            true
         );
     }
 
@@ -683,6 +718,10 @@ public:
         return m_staticFunctions;
     }
 
+    const std::map<std::string, Function*>& getClassMethods() const {
+        return m_classMethods;
+    }
+
     const std::map<std::string, PyObject*>& getClassMembers() const {
         return m_classMembers;
     }
@@ -699,6 +738,10 @@ public:
         return m_own_staticFunctions;
     }
 
+    const std::map<std::string, Function*>& getOwnClassMethods() const {
+        return m_own_classMethods;
+    }
+
     const std::map<std::string, PyObject*>& getOwnClassMembers() const {
         return m_own_classMembers;
     }
@@ -733,32 +776,97 @@ public:
         return m_hasDelAttrMagicMethod;
     }
 
-    void initializeMRO() {
-        // this is not how the MRO actually works, but we have yet to actually
-        // code it correctly.
-        std::set<HeldClass*> seen;
+    // fill out m_mro according to C3 precedence rules
+    void _computeMroSequence() {
+        m_mro.push_back(this);
 
-        std::function<void (HeldClass*)> visit = [&](HeldClass* cls) {
-            if (seen.find(cls) == seen.end()) {
-                seen.insert(cls);
+        std::vector<std::vector<HeldClass*> > chains;
+        std::vector<std::set<HeldClass*> > chainTails;
 
-                m_mro.push_back(cls);
-                m_ancestor_to_mro_index[cls] = m_mro.size() - 1;
-
-                for (auto b: cls->getBases()) {
-                    visit(b);
-                }
-            }
-        };
-
-        visit(this);
-
-        if (m_ancestor_to_mro_index.find(this) == m_ancestor_to_mro_index.end() ||
-                m_ancestor_to_mro_index.find(this)->second != 0) {
-            throw std::runtime_error("Somehow " + m_name + " doesn't have itself as MRO 0");
+        for (HeldClass* base: m_bases) {
+            chains.push_back(base->getMro());
         }
 
-        // build our own method resolution table directly from our parents.
+        chains.push_back(m_bases);
+        chainTails.resize(chains.size());
+
+        for (size_t i = 0; i < chains.size(); i++) {
+            for (size_t j = 1; j < chains[i].size(); j++) {
+                chainTails[i].insert(chains[i][j]);
+            }
+        }
+
+        // check if 'h' is in one of the linearization tails. If it is
+        // then we can't use it
+        auto isValid = [&](HeldClass* h) {
+            for (auto& ct: chainTails) {
+                if (ct.find(h) != ct.end()) {
+                    return false;
+                }
+            }
+
+            return true;
+        };
+
+        auto pickCls = [&]() {
+            bool anyPopulated = false;
+
+            for (auto& chain: chains) {
+                if (chain.size() && isValid(chain[0])) {
+                    return chain[0];
+                }
+
+                if (chain.size()) {
+                    anyPopulated = true;
+                }
+            }
+
+            if (anyPopulated) {
+                throw std::runtime_error(
+                    "Class MRO order for " + name() + " couldn't be determined. No "
+                    "valid linearization exists."
+                );
+            }
+
+            return (HeldClass*)nullptr;
+        };
+
+        while (true) {
+            HeldClass* hc = pickCls();
+
+            if (!hc) {
+                return;
+            }
+
+            m_mro.push_back(hc);
+
+            for (size_t i = 0; i < chains.size(); i++) {
+                if (chains[i].size() && chains[i][0] == hc) {
+                    chains[i].erase(chains[i].begin(), chains[i].begin() + 1);
+                    chainTails[i].erase(hc);
+
+                    if (chains[i].size()) {
+                        chainTails[i].erase(chains[i][0]);
+                    }
+                }
+            }
+        }
+    }
+
+    // every HeldClass has a set of functions that are defined within that specific
+    // class body, and then they have the fully formed versions of those functions that
+    // inherit and overload the versions defined in their base classes.
+    // this function builds the versions we actually call from our own and our parent
+    // versions.
+
+    // this function can be called multiple times, which happens after the class is
+    // deserialized, since we may be rebuilding the function globals.
+    void mergeOwnFunctionsIntoInheritanceTree() {
+        m_classMembers.clear();
+        m_memberFunctions.clear();
+        m_classMethods.clear();
+        m_propertyFunctions.clear();
+
         for (HeldClass* base: m_mro) {
             for (auto nameAndObj: base->m_own_classMembers) {
                 if (m_classMembers.find(nameAndObj.first) == m_classMembers.end()) {
@@ -768,8 +876,25 @@ public:
 
             mergeInto(m_memberFunctions, base->m_own_memberFunctions);
             mergeInto(m_staticFunctions, base->m_own_staticFunctions);
+            mergeInto(m_classMethods, base->m_own_classMethods);
             mergeInto(m_propertyFunctions, base->m_own_propertyFunctions);
         }
+    }
+
+    void initializeMRO() {
+        _computeMroSequence();
+
+        for (size_t i = 0; i < m_mro.size(); i++) {
+            m_ancestor_to_mro_index[m_mro[i]] = i;
+        }
+
+        if (m_ancestor_to_mro_index.find(this) == m_ancestor_to_mro_index.end() ||
+                m_ancestor_to_mro_index.find(this)->second != 0) {
+            throw std::runtime_error("Somehow " + m_name + " doesn't have itself as MRO 0");
+        }
+
+        // build our own method resolution table directly from our parents.
+        mergeOwnFunctionsIntoInheritanceTree();
 
         std::set<std::string> membersSoFar;
 
@@ -887,29 +1012,6 @@ public:
             if (it == target.end()) {
                 target[nameAndFunc.first] = nameAndFunc.second;
             } else {
-                // we need to check that the function signatures we produce can obey the
-                // return types dictated by the base class. This means that any function
-                // already in 'target' (the subclass methods we've already accepted) that
-                // overlaps with a method in the base class (meaning that a dispatch to the
-                // child could also dispatch to the base) must have a return type that is
-                // _more_ precise than the base class, so that we can ensure that when we
-                // compile the child class to masquerade as the base class we are able
-                // to comply with the return-type assumptions made by callers.
-                for (auto childOverload: target[nameAndFunc.first]->getOverloads()) {
-                    for (auto baseOverload: nameAndFunc.second->getOverloads()) {
-                        if (!childOverload.disjointFrom(baseOverload)) {
-                            if (childOverload.getReturnType() != baseOverload.getReturnType()) {
-                                // we should really be checking for 'coverage' but for now we just
-                                // check for type equality
-                                throw std::runtime_error(
-                                    "Overloads of '" + nameAndFunc.first + "' don't have the same return type:\n" +
-                                    "    " + childOverload.toString() + "\n" +
-                                    "    " + baseOverload.toString()
-                                );
-                            }
-                        }
-                    }
-                }
                 target[nameAndFunc.first] = Function::merge(target[nameAndFunc.first], nameAndFunc.second);
             }
         }
@@ -999,6 +1101,8 @@ private:
 
     std::map<std::string, Function*> m_propertyFunctions;
 
+    std::map<std::string, Function*> m_classMethods;
+
     std::map<std::string, PyObject*> m_classMembers;
 
     // the original members we were provided with
@@ -1012,6 +1116,8 @@ private:
 
     std::map<std::string, PyObject*> m_own_classMembers;
 
+    std::map<std::string, Function*> m_own_classMethods;
+
     std::unordered_map<const char*, size_t, HashConstCharPtr, ConstCharPtrsAreEqual> m_membersByName;
 
     std::unordered_map<const char*, BoundMethod*, HashConstCharPtr, ConstCharPtrsAreEqual> m_memberFunctionMethodTypes[2];
