@@ -192,7 +192,9 @@ public:
                             o.getFunctionClosureVarnames(),
                             o.getClosureVariableBindings(),
                             copyType(o.getReturnType()),
-                            args
+                            o.getSignatureFunction(),
+                            args,
+                            o.getMethodOf()
                         )
                     );
                 }
@@ -232,6 +234,7 @@ public:
                 std::map<std::string, Function*> staticFunctions;
                 std::map<std::string, Function*> propertyFunctions;
                 std::map<std::string, PyObject*> classMembers;
+                std::map<std::string, Function*> classMethods;
 
                 for (auto& base: c->getBases()) {
                     bases.push_back((Class*)copyType(base->getClassType()));
@@ -243,6 +246,9 @@ public:
                 for (auto& nameAndF: c->getOwnStaticFunctions()) {
                     staticFunctions[nameAndF.first] = (Function*)copyType(nameAndF.second);
                 }
+                for (auto& nameAndF: c->getOwnClassMethods()) {
+                    classMethods[nameAndF.first] = (Function*)copyType(nameAndF.second);
+                }
                 for (auto& nameAndF: c->getOwnPropertyFunctions()) {
                     propertyFunctions[nameAndF.first] = (Function*)copyType(nameAndF.second);
                 }
@@ -258,7 +264,11 @@ public:
                     memberFunctions,
                     staticFunctions,
                     propertyFunctions,
-                    classMembers
+                    classMembers,
+                    classMethods,
+                    // we want to ensure the class has appropriate
+                    // methodOf for all of its function types
+                    true
                 );
 
                 forwardC->define(outC);
@@ -375,6 +385,10 @@ public:
                 decref(dest->prop_del);
                 decref(dest->prop_doc);
 
+                #if PY_MINOR_VERSION >= 10
+                decref(dest->prop_name);
+                #endif
+
                 mObjectMemo[obj] = res;
 
                 if (source->prop_get) {
@@ -401,6 +415,14 @@ public:
                     dest->prop_doc = nullptr;
                 }
 
+                #if PY_MINOR_VERSION >= 10
+                if (source->prop_name) {
+                    dest->prop_name = incref(copy(source->prop_name).pyobj());
+                } else {
+                    dest->prop_name = nullptr;
+                }
+                #endif
+
                 dest->getter_doc = source->getter_doc;
 
                 return PyObjectHandle::steal(res);
