@@ -112,6 +112,19 @@ public:
         throw std::runtime_error("Can't serialize populated Pointers");
     }
 
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        copy_constructor(dest, src);
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        // we don't follow pointers since we can't be sure they reference valid data.
+        return 0;
+    }
+
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t wireType) {
         buffer.readUnsignedVarint();
