@@ -592,6 +592,11 @@ PyObject* PyClassInstance::tpGetattrGeneric(
     {
         auto it = heldClass->getClassMembers().find(attrName);
         if (it != heldClass->getClassMembers().end()) {
+            // if this follows the descriptor protocol, use that
+            if (PyObject_HasAttrString(it->second, "__get__")) {
+                return PyObject_CallMethod(it->second, "__get__", "OO", self, (PyObject*)self->ob_type);
+            }
+
             return incref(it->second);
         }
     }
@@ -735,7 +740,11 @@ void PyClassInstance::mirrorTypeInformationIntoPyTypeConcrete(Class* classT, PyT
 
     PyDict_SetItemString(pyType->tp_dict, "ClassMembers", classMembers);
 
+    PyObjectStealer staticMemberFunctions(PyDict_New());
+    PyDict_SetItemString(pyType->tp_dict, "StaticMemberFunctions", staticMemberFunctions);
     for (auto nameAndObj: classT->getStaticFunctions()) {
+        PyDict_SetItemString(staticMemberFunctions, nameAndObj.first.c_str(), typePtrToPyTypeRepresentation(nameAndObj.second));
+
         if (nameAndObj.second->getClosureType()->bytecount()) {
             throw std::runtime_error(
                 "Somehow, " + classT->name() + "."
@@ -753,6 +762,66 @@ void PyClassInstance::mirrorTypeInformationIntoPyTypeConcrete(Class* classT, PyT
             )
         );
     }
+
+    PyObjectStealer classMemberFunctions(PyDict_New());
+    PyDict_SetItemString(pyType->tp_dict, "ClassMemberFunctions", classMemberFunctions);
+    for (auto nameAndObj: classT->getClassMethods()) {
+        PyDict_SetItemString(classMemberFunctions, nameAndObj.first.c_str(), typePtrToPyTypeRepresentation(nameAndObj.second));
+
+        if (nameAndObj.second->getClosureType()->bytecount()) {
+            throw std::runtime_error(
+                "Somehow, " + classT->name() + "."
+                + nameAndObj.first + " has a populated closure."
+            );
+        }
+
+        PyDict_SetItemString(
+            pyType->tp_dict,
+            nameAndObj.first.c_str(),
+            PyClassMethod_New(
+                PyInstance::initialize(nameAndObj.second, [&](instance_ptr data){
+                    //nothing to do - functions like this are just types.
+                })
+            )
+        );
+    }
+
+    if (!asHeldClass) {
+        // mirror the MRO + the hierarchy above into the actual __mro__ variable, which
+        // over the long run should replace MRO and Bases.
+        // we have to include the base classes above us (Class, Type, object)
+        // so that standard python object model behaviors work.
+        PyObjectStealer realMro(PyTuple_New(classT->getHeldClass()->getMro().size() + 3));
+
+        long k = 0;
+        for (;k < classT->getHeldClass()->getMro().size(); k++) {
+            PyTuple_SetItem(
+                realMro,
+                k,
+                incref(typePtrToPyTypeRepresentation(classT->getHeldClass()->getMro()[k]->getClassType()))
+            );
+        }
+
+        PyTuple_SetItem(
+            realMro,
+            k++,
+            incref((PyObject*)pyType->tp_base)
+        );
+
+        PyTuple_SetItem(
+            realMro,
+            k++,
+            incref((PyObject*)pyType->tp_base->tp_base)
+        );
+
+        PyTuple_SetItem(
+            realMro,
+            k++,
+            incref((PyObject*)pyType->tp_base->tp_base->tp_base)
+        );
+
+        pyType->tp_mro = incref(realMro);
+    }
 }
 
 int PyClassInstance::tp_setattr_concrete(PyObject* attrName, PyObject* attrVal) {
