@@ -43,6 +43,7 @@
 #include "PyPythonSubclassInstance.hpp"
 #include "PyPythonObjectOfTypeInstance.hpp"
 #include "PyOneOfInstance.hpp"
+#include "PySubclassOfInstance.hpp"
 #include "PyForwardInstance.hpp"
 #include "PyEmbeddedMessageInstance.hpp"
 #include "PyPyCellInstance.hpp"
@@ -367,6 +368,7 @@ PyObject* PyInstance::tp_new_type(PyTypeObject *subtype, PyObject *args, PyObjec
     if (catToProduce == Type::TypeCategory::catFunction ) { return MakeFunctionType(nullptr, args); }
     if (catToProduce == Type::TypeCategory::catClass ) { return MakeClassType(nullptr, args); }
     if (catToProduce == Type::TypeCategory::catAlternative ) { return MakeAlternativeType(nullptr, args, kwds); }
+    if (catToProduce == Type::TypeCategory::catSubclassOf ) { return MakeSubclassOfType(nullptr, args); }
 
     PyErr_Format(PyExc_TypeError, "unknown TypeCategory %S", (PyObject*)subtype);
     return NULL;
@@ -401,8 +403,16 @@ PyObject* PyInstance::tp_new(PyTypeObject *subtype, PyObject *args, PyObject *kw
 
             // not reachable
             assert(false);
-
         } else {
+            if (eltType->isClass() && ((Class*)eltType)->getOwnClassMembers().find("__typed_python_template__")
+                    != ((Class*)eltType)->getOwnClassMembers().end()) {
+                return PyObject_Call(
+                    ((Class*)eltType)->getOwnClassMembers().find("__typed_python_template__")->second,
+                    args,
+                    kwds
+                );
+            }
+
             Instance inst(eltType, [&](instance_ptr tgt) {
                 constructFromPythonArguments(tgt, eltType, args, kwds);
             });
@@ -1044,8 +1054,12 @@ PyTypeObject* PyInstance::allTypesBaseType() {
 
             #if PY_MINOR_VERSION >= 8
             .tp_vectorcall = 0,
+            #endif
+
+            #if PY_MINOR_VERSION == 8
             .tp_print = 0,
             #endif
+
         };
 
         PyType_Ready(result);
@@ -1064,7 +1078,7 @@ PyTypeObject* PyInstance::typeCategoryBaseType(Type::TypeCategory category) {
 
     if (types.find(category) == types.end()) {
         PyObject* classDict = PyDict_New();
-        PyDict_SetItemString(classDict, "__typed_python_module__", PyUnicode_FromString("typed_python._types"));
+        PyDict_SetItemString(classDict, "__module__", PyUnicode_FromString("typed_python._types"));
 
         types[category] = new NativeTypeCategoryWrapper { {
             PyVarObject_HEAD_INIT(NULL, 0)              // TYPE (c.f., Type Objects)
@@ -1123,8 +1137,12 @@ PyTypeObject* PyInstance::typeCategoryBaseType(Type::TypeCategory category) {
             .tp_version_tag = 0,                        // unsigned int
             .tp_finalize = 0,                           // destructor
 
+
             #if PY_MINOR_VERSION >= 8
             .tp_vectorcall = 0,
+            #endif
+
+            #if PY_MINOR_VERSION == 8
             .tp_print = 0,
             #endif
             },
@@ -1179,7 +1197,7 @@ PyTypeObject* PyInstance::typeObjInternal(Type* inType) {
 
     types[inType] = new NativeTypeWrapper { {
             PyVarObject_HEAD_INIT(NULL, 0)              // TYPE (c.f., Type Objects)
-            .tp_name = (new std::string(inType->name()))->c_str(),          // const char*
+            .tp_name = (new std::string(inType->nameWithModule()))->c_str(),          // const char*
             .tp_basicsize = sizeof(PyInstance),         // Py_ssize_t
             .tp_itemsize = 0,                           // Py_ssize_t
             .tp_dealloc = PyInstance::tp_dealloc,       // destructor
@@ -1244,8 +1262,12 @@ PyTypeObject* PyInstance::typeObjInternal(Type* inType) {
             .tp_version_tag = 0,                        // unsigned int
             .tp_finalize = 0,                           // destructor
 
+
             #if PY_MINOR_VERSION >= 8
             .tp_vectorcall = 0,
+            #endif
+
+            #if PY_MINOR_VERSION == 8
             .tp_print = 0,
             #endif
             },
@@ -1610,6 +1632,7 @@ PyObject* PyInstance::categoryToPyString(Type::TypeCategory cat) {
     if (cat == Type::TypeCategory::catPyCell) { static PyObject* res = PyUnicode_FromString("PyCell"); return res; }
     if (cat == Type::TypeCategory::catTypedCell) { static PyObject* res = PyUnicode_FromString("TypedCell"); return res; }
     if (cat == Type::TypeCategory::catPythonObjectOfType) { static PyObject* res = PyUnicode_FromString("PythonObjectOfType"); return res; }
+    if (cat == Type::TypeCategory::catSubclassOf) { static PyObject* res = PyUnicode_FromString("SubclassOf"); return res; }
 
     static PyObject* res = PyUnicode_FromString("Unknown");
     return res;
@@ -1827,6 +1850,6 @@ Type* PyInstance::unwrapTypeArgToTypePtr(PyObject* typearg) {
         return valueType;
     }
 
-    PyErr_Format(PyExc_TypeError, "Cannot convert to a native type.");
+    PyErr_Format(PyExc_TypeError, "Cannot convert %R to a typed_python Type", typearg);
     return NULL;
 }
