@@ -258,53 +258,23 @@ void PythonSerializationContext::serializePythonObjectNamedOrAsObj(PyObject* o,
 
     b.writeUnsignedVarintObject(FieldNumbers::MEMO, b.cachePointer(o).first);
 
-    //check whether this is a type derived from a serializable native type, which we don't support
-    if (PyLong_Check(o)) {
-        throwDerivedClassError("long");
-    }
-    if (PyFloat_Check(o)) {
-        throwDerivedClassError("float");
-    }
-    if (PyBytes_Check(o)) {
-        throwDerivedClassError("bytes");
-    }
-    if (PyComplex_Check(o)) {
-        throwDerivedClassError("complex");
-    }
-    if (PyUnicode_Check(o)) {
-        throwDerivedClassError("str");
-    }
-    if (PyList_Check(o)) {
-        throwDerivedClassError("list");
-    }
-    if (PyTuple_Check(o)) {
-        throwDerivedClassError("tuple");
-    }
-    if (PySet_Check(o)) {
-        throwDerivedClassError("set");
-    }
-    if (PyFrozenSet_Check(o)) {
-        throwDerivedClassError("frozenset");
-    }
-    if (PyDict_Check(o)) {
-        throwDerivedClassError("dict");
-    }
-    if (PyModule_Check(o)) {
-        throw std::runtime_error(
-            std::string("Cannot serialize module '")
-            + PyModule_GetName(o)
-            + "'. This code should be unreachable."
-        );
-    }
-
     b.writeBeginCompound(FieldNumbers::OBJECT_TYPEANDDICT);
     serializePythonObject((PyObject*)o->ob_type, b, 0);
     PyObjectStealer objDict(PyObject_GenericGetDict(o, nullptr));
     if (!objDict) {
-        PyErr_Format(PyExc_TypeError,
-            "Object %S (of type %S) had no dict", o, o->ob_type
-        );
-
+        if (PyCFunction_Check(o)) {
+            PyErr_Format(PyExc_TypeError,
+                "Method %s (of instance %S) had no dict.",
+                ((PyCFunctionObject*)o)->m_ml ? ((PyCFunctionObject*)o)->m_ml->ml_name : "<NULL>",
+                PyCFunction_GetSelf(o) ? (PyObject*)PyCFunction_GetSelf(o) : Py_None
+            );
+        } else {
+            PyErr_Format(PyExc_TypeError,
+                "Object %S (of type %S) had no dict.",
+                o,
+                o->ob_type
+            );
+        }
         throw PythonExceptionSet();
     }
     serializePythonObject(objDict, b, 1);
@@ -314,21 +284,19 @@ void PythonSerializationContext::serializePythonObjectNamedOrAsObj(PyObject* o,
 void PythonSerializationContext::serializePythonObjectRepresentation(PyObject* representation, SerializationBuffer& b, size_t fieldNumber) const {
     PyEnsureGilAcquired acquireTheGil;
 
-    if (!PyTuple_Check(representation) || PyTuple_Size(representation) != 3) {
-        throw std::runtime_error("representationFor should return None or a tuple with 3 things");
+    if (!PyTuple_Check(representation) || PyTuple_Size(representation) < 2 
+            || PyTuple_Size(representation) > 6) {
+        throw std::runtime_error("representationFor should return None or a tuple with 2 to 6 things");
     }
     if (!PyTuple_Check(PyTuple_GetItem(representation, 1))) {
         throw std::runtime_error("representationFor second arguments should be a tuple");
     }
 
-    PyObjectHolder rep0(PyTuple_GetItem(representation, 0));
-    PyObjectHolder rep1(PyTuple_GetItem(representation, 1));
-    PyObjectHolder rep2(PyTuple_GetItem(representation, 2));
-
     b.writeBeginCompound(fieldNumber);
-    serializePythonObject(rep0, b, 0);
-    serializePythonObject(rep1, b, 1);
-    serializePythonObject(rep2, b, 2);
+    for (long ix = 0; ix < PyTuple_Size(representation); ix++) {
+        PyObjectHolder rep(PyTuple_GetItem(representation, ix));
+        serializePythonObject(rep, b, ix);
+    }
     b.writeEndCompound();
 }
 
@@ -524,8 +492,9 @@ void PythonSerializationContext::serializeMutuallyRecursiveTypeGroup(MutuallyRec
                     }
 
                     if (representation != Py_None) {
-                        if (!PyTuple_Check(representation) || PyTuple_Size(representation) != 3) {
-                            throw std::runtime_error("representationFor should return None or a tuple with 3 things");
+                        if (!PyTuple_Check(representation) || PyTuple_Size(representation) < 2 
+                                || PyTuple_Size(representation) > 6) {
+                            throw std::runtime_error("representationFor should return None or a tuple with 2 to 6 things");
                         }
                         if (!PyTuple_Check(PyTuple_GetItem(representation, 1))) {
                             throw std::runtime_error("representationFor second arguments should be a tuple");
@@ -539,7 +508,9 @@ void PythonSerializationContext::serializeMutuallyRecursiveTypeGroup(MutuallyRec
                         serializePythonObject(PyTuple_GetItem(representation, 0), b, 1);
                         serializePythonObject(PyTuple_GetItem(representation, 1), b, 2);
 
-                        indicesWrittenAsObjectAndRep[index].set(PyTuple_GetItem(representation, 2));
+                        indicesWrittenAsObjectAndRep[index].steal(
+                            PyTuple_GetSlice(representation, 2, PyTuple_Size(representation))
+                        );
                     } else
                     if (PyTuple_Check(obj.pyobj())) {
                         b.writeUnsignedVarintObject(0, 5);
@@ -610,11 +581,21 @@ void PythonSerializationContext::serializeMutuallyRecursiveTypeGroup(MutuallyRec
                         PyObjectStealer objDict(PyObject_GenericGetDict(obj.pyobj(), nullptr));
 
                         if (!objDict) {
-                            PyErr_Format(PyExc_TypeError,
-                                "Object %S (of type %S) had no dict.",
-                                obj.pyobj(),
-                                obj.pyobj()->ob_type
-                            );
+                            if (PyCFunction_Check(obj.pyobj())) {
+                                auto o = obj.pyobj();
+
+                                PyErr_Format(PyExc_TypeError,
+                                    "Method %s (of instance %S) had no dict.",
+                                    ((PyCFunctionObject*)o)->m_ml ? ((PyCFunctionObject*)o)->m_ml->ml_name : "<NULL>",
+                                    PyCFunction_GetSelf(o) ? (PyObject*)PyCFunction_GetSelf(o) : Py_None
+                                );
+                            } else {
+                                PyErr_Format(PyExc_TypeError,
+                                    "Object %S (of type %S) had no dict.",
+                                    obj.pyobj(),
+                                    obj.pyobj()->ob_type
+                                );
+                            }
 
                             throw PythonExceptionSet();
                         }
@@ -795,6 +776,8 @@ void PythonSerializationContext::serializeNativeTypeInner(
         serializeNativeType(((PointerTo*)nativeType)->getEltType(), b, 1);
     } else if (nativeType->getTypeCategory() == Type::TypeCategory::catRefTo) {
         serializeNativeType(((RefTo*)nativeType)->getEltType(), b, 1);
+    } else if (nativeType->getTypeCategory() == Type::TypeCategory::catSubclassOf) {
+        serializeNativeType(((SubclassOfType*)nativeType)->getSubclassOf(), b, 1);
     } else if (nativeType->getTypeCategory() == Type::TypeCategory::catBoundMethod) {
         b.writeStringObject(1, ((BoundMethod*)nativeType)->getFuncName());
         serializeNativeType(((BoundMethod*)nativeType)->getFirstArgType(), b, 2);
@@ -864,6 +847,7 @@ void PythonSerializationContext::serializeNativeTypeInner(
         //methods
         serializeClassFunDict(cls->getOwnMemberFunctions(), b, 3);
         serializeClassFunDict(cls->getOwnStaticFunctions(), b, 4);
+        serializeClassFunDict(cls->getOwnClassMethods(), b, 9);
         serializeClassFunDict(cls->getOwnPropertyFunctions(), b, 5);
         serializeClassClassMemberDict(cls->getOwnClassMembers(), b, 6);
 
