@@ -99,6 +99,18 @@ public:
         m_base->serialize(self, buffer, fieldNumber);
     }
 
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        m_base->deepcopy(dest, src, context);
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        return m_base->deepBytecount(instance, alreadyVisited, outSlabs);
+    }
+
     template<class buf_t>
     void deserialize(instance_ptr self, buf_t& buffer, size_t wireType) {
         m_base->deserialize(self, buffer, wireType);
@@ -110,6 +122,10 @@ public:
         return m_base->cmp(left, right, pyComparisonOp, suppressExceptions);
     }
 
+    bool isPODConcrete() {
+        return m_base->isPOD();
+    }
+
     void constructor(instance_ptr self) {
         m_base->constructor(self);
     }
