@@ -20,13 +20,11 @@ from typed_python.python_ast import (
 )
 from typed_python.hash import sha_hash
 from typed_python.compiler.runtime_lock import runtimeLock
-from typed_python.type_function import ConcreteTypeFunction, reconstructTypeFunctionType, isTypeFunctionType
+from typed_python.type_function import TypeFunction, reconstructTypeFunctionType, isTypeFunctionType
 from types import FunctionType, ModuleType, CodeType, MethodType, BuiltinFunctionType
 from _thread import LockType, RLock
 import abc
-import numpy
 import sys
-import datetime
 import importlib
 import threading
 import types
@@ -108,6 +106,9 @@ DEFAULT_NAME_TO_OVERRIDE = {
     ".builtin.module_type": ModuleType,
     ".builtin.method_type": MethodType,
     ".builtin.function_type": FunctionType,
+    ".builtin.ellipsis_type": type(...),
+    ".builtin.ellipsis": ...,
+    ".builtin.NotImplemented": NotImplemented
 }
 
 
@@ -258,11 +259,12 @@ class SerializationContext:
             if tName is not None:
                 return ".alt." + tName + ":" + str(t.Index)
 
-        if isinstance(t, ConcreteTypeFunction):
-            name = t._concreteTypeFunction.__module__ + "." + t._concreteTypeFunction.__name__
-            if self.objectFromName(name, allowImport=False) is t:
-                return name
-            return None
+        if isinstance(t, type) and issubclass(t, TypeFunction):
+            if len(t.MRO) == 2:
+                name = t.__module__ + "." + t.__name__
+                if self.objectFromName(name, allowImport=False) is t:
+                    return name
+                return None
 
         if isinstance(t, dict) and '__name__' in t and isinstance(t['__name__'], str):
             maybeModule = self.objectFromName('.modules.' + t['__name__'], allowImport=False)
@@ -270,8 +272,8 @@ class SerializationContext:
                 return ".module_dict." + t['__name__']
 
         if isinstance(t, (FunctionType, BuiltinFunctionType)) or isinstance(t, type) and issubclass(t, _types.Function):
-            mname = getattr(t, "__typed_python_module__", t.__module__)
-            qualname = getattr(t, "__typed_python_qualname__", t.__qualname__)
+            mname = getattr(t, "__module__", t.__module__)
+            qualname = getattr(t, "__qualname__", t.__qualname__)
             fname = t.__name__
 
             if mname is not None:
@@ -289,7 +291,7 @@ class SerializationContext:
                     return ".typeof." + name
 
         elif isinstance(t, type) and issubclass(t, Alternative):
-            mname = t.__typed_python_module__
+            mname = t.__module__
             fname = t.__name__
 
             if self.objectFromName(mname + "." + fname, allowImport=False) is t:
@@ -302,11 +304,6 @@ class SerializationContext:
         elif isinstance(t, type):
             fname = t.__name__
 
-            if hasattr(t, "__typed_python_module__"):
-                mname = getattr(t, "__typed_python_module__", None)
-                if self.objectFromName(mname + "." + fname, allowImport=False) is t:
-                    return mname + "." + fname
-
             if getattr(t, "__typed_python_category__", None) == 'Value':
                 return None
 
@@ -467,15 +464,21 @@ class SerializationContext:
             @param inst: an instance to be serialized
             @return a representation object or None
         '''
+        if type(inst) in (tuple, list, dict, set, str, int, bool, float):
+            return None
+
+        if isinstance(inst, CellType):
+            return None
+
         if isinstance(inst, types.ModuleType) and inst is not sys.modules.get(inst.__name__):
             # this is an 'unnamed' module
             return (types.ModuleType, (inst.__name__,), inst.__dict__)
 
         if isinstance(inst, LockType):
-            return (threading.Lock, (), None)
+            return (threading.Lock, ())
 
         if isinstance(inst, RLock):
-            return (threading.RLock, (), None)
+            return (threading.RLock, ())
 
         if isinstance(inst, type):
             # only serialize Class and Alternative objects from type functions.
@@ -505,7 +508,10 @@ class SerializationContext:
                     getset_descriptor = type(type.__dict__['__doc__'])
                     wrapper_descriptor = type(object.__repr__)
 
-                    if name != "__dict__" and not isinstance(memb, (wrapper_descriptor, getset_descriptor)):
+                    if (
+                        name not in ("__dict__", '__slotnames__')
+                        and not isinstance(memb, (wrapper_descriptor, getset_descriptor))
+                    ):
                         classMembers[name] = memb
 
                 # filter out weird class members introduced by 'abc'
@@ -527,30 +533,6 @@ class SerializationContext:
         if isinstance(inst, classmethod):
             return (classmethod, (None,), inst.__func__)
 
-        if isinstance(inst, numpy.ndarray):
-            return inst.__reduce__()
-
-        if isinstance(inst, numpy.number):
-            return inst.__reduce__() + (None,)
-
-        if isinstance(inst, numpy.dtype):
-            return (numpy.dtype, (str(inst),), None)
-
-        if isinstance(inst, datetime.datetime):
-            return inst.__reduce__() + (None,)
-
-        if isinstance(inst, datetime.date):
-            return inst.__reduce__() + (None,)
-
-        if isinstance(inst, datetime.time):
-            return inst.__reduce__() + (None,)
-
-        if isinstance(inst, datetime.timedelta):
-            return inst.__reduce__() + (None,)
-
-        if isinstance(inst, datetime.tzinfo):
-            return inst.__reduce__() + (None,)
-
         if isinstance(inst, CodeType):
             pyast = convertFunctionToAlgebraicPyAst(
                 inst,
@@ -575,7 +557,7 @@ class SerializationContext:
                     inst.co_filename if self.encodeLineInformationForCode else "",
                     inst.co_name,
                     inst.co_firstlineno if self.encodeLineInformationForCode else 0,
-                    inst.co_lnotab
+                    inst.co_lnotab if sys.version_info.minor < 10 else inst.co_linetable
                 ) + (
                     () if sys.version_info.minor < 8 or inst.co_posonlyargcount == 0 else
                     (inst.co_posonlyargcount,)
@@ -587,7 +569,11 @@ class SerializationContext:
             representation = {}
             representation["qualname"] = inst.__qualname__
             representation["name"] = inst.__name__
-            representation["module"] = inst.__module__
+            if self.encodeLineInformationForCode:
+                representation["module"] = inst.__module__
+            else:
+                representation["module"] = ""
+
             representation["annotations"] = inst.__annotations__
             representation["defaults"] = inst.__defaults__
             representation["kwdefaults"] = inst.__kwdefaults__
@@ -628,18 +614,44 @@ class SerializationContext:
 
             return (createFunctionWithLocalsAndGlobals, args, representation)
 
+        if not isinstance(inst, type) and hasattr(type(inst), '__reduce_ex__'):
+            res = inst.__reduce_ex__(4)
+
+            # pickle supports a protocol where __reduce__ can return a string
+            # giving a global name. We'll already find that separately, so we
+            # don't want to handle it here. We ought to look at this in more detail
+            # however
+            if isinstance(res, str):
+                return None
+
+            return res
+
+        if not isinstance(inst, type) and hasattr(type(inst), '__reduce__'):
+            res = inst.__reduce__()
+
+            # pickle supports a protocol where __reduce__ can return a string
+            # giving a global name. We'll already find that separately, so we
+            # don't want to handle it here. We ought to look at this in more detail
+            # however
+            if isinstance(res, str):
+                return None
+
+            return res
+
         return None
 
-    def setInstanceStateFromRepresentation(self, instance, representation):
+    def setInstanceStateFromRepresentation(
+        self, instance, representation=None, itemIt=None, kvPairIt=None, setStateFun=None
+    ):
         if representation is reconstructTypeFunctionType:
-            return True
+            return
 
         if isinstance(instance, types.MethodType):
             _types.setMethodObjectInternals(instance, representation[0], representation[1])
-            return True
+            return
 
         if isinstance(instance, (LockType, RLock)):
-            return True
+            return
 
         if isinstance(instance, types.ModuleType):
             # note that we can't simply copy the contents of the dict into
@@ -647,47 +659,22 @@ class SerializationContext:
             # dict itself as their 'globals', and copying directly would break
             # that relationship.
             _types.setModuleDict(instance, representation)
-            return True
+            return
 
         if isinstance(instance, property):
             _types.setPropertyGetSetDel(instance, representation[0], representation[1], representation[2])
-            return True
+            return
 
         if isinstance(instance, staticmethod):
             _types.setClassOrStaticmethod(instance, representation)
-            return True
+            return
 
         if isinstance(instance, classmethod):
             _types.setClassOrStaticmethod(instance, representation)
-            return True
+            return
 
         if isinstance(instance, CodeType):
-            return True
-
-        if isinstance(instance, datetime.datetime):
-            return True
-
-        if isinstance(instance, datetime.date):
-            return True
-
-        if isinstance(instance, datetime.time):
-            return True
-
-        if isinstance(instance, datetime.timedelta):
-            return True
-
-        if isinstance(instance, datetime.tzinfo):
-            return True
-
-        if isinstance(instance, numpy.dtype):
-            return True
-
-        if isinstance(instance, numpy.number):
-            return True
-
-        if isinstance(instance, numpy.ndarray):
-            instance.__setstate__(representation)
-            return True
+            return
 
         if isinstance(instance, FunctionType):
             instance.__name__ = representation['name']
@@ -705,16 +692,28 @@ class SerializationContext:
 
             _types.setFunctionClosure(instance, representation['closure'])
 
-            return True
+            return
 
         if isinstance(instance, type):
             # set class members
             if representation is not None:
                 for k, v in representation.items():
                     setattr(instance, k, v)
-            return True
+            return
 
         if isinstance(instance, ModuleType):
-            return True
+            return
 
-        return False
+        if setStateFun is not None:
+            setStateFun(instance, representation)
+        elif hasattr(type(instance), '__setstate__') and representation is not None:
+            type(instance).__setstate__(instance, representation)
+        elif representation is not None:
+            instance.__dict__.update(representation)
+
+        if itemIt:
+            instance.extend(itemIt)
+
+        if kvPairIt:
+            for key, val in kvPairIt:
+                instance[key] = val
