@@ -72,6 +72,102 @@ public:
         });
     }
 
+    void deepcopyConcrete(
+        instance_ptr dest,
+        instance_ptr src,
+        DeepcopyContext& context
+    ) {
+        layout_ptr& destLayout = *(layout**)dest;
+        layout_ptr& srcLayout = *(layout**)src;
+
+        if (!srcLayout) {
+            destLayout = srcLayout;
+            return;
+        }
+
+        auto it = context.alreadyAllocated.find((instance_ptr)srcLayout);
+        if (it == context.alreadyAllocated.end()) {
+            destLayout = (layout_ptr)context.slab->allocate(sizeof(layout), this);
+            destLayout->hash_cache = srcLayout->hash_cache;
+            destLayout->refcount = 0;
+
+            size_t reserveCount = srcLayout->count;
+
+            if (reserveCount == 0 && !m_is_tuple) {
+                reserveCount = 1;
+            }
+
+            destLayout->reserved = reserveCount;
+            destLayout->count = srcLayout->count;
+
+            if (reserveCount) {
+                destLayout->data = (instance_ptr)context.slab->allocate(getEltType()->bytecount() * reserveCount, nullptr);
+            } else {
+                destLayout->data = nullptr;
+            }
+
+            if (destLayout->count) {
+                if (getEltType()->isPOD()) {
+                    memcpy(destLayout->data, srcLayout->data, getEltType()->bytecount() * srcLayout->count);
+                } else {
+                    for (long k = 0; k < srcLayout->count; k++) {
+                        m_element_type->deepcopy(
+                            destLayout->data + k * m_element_type->bytecount(),
+                            srcLayout->data + k * m_element_type->bytecount(),
+                            context
+                        );
+                    }
+                }
+            }
+
+            context.alreadyAllocated[(instance_ptr)srcLayout] = (instance_ptr)destLayout;
+        } else {
+            destLayout = (layout_ptr)it->second;
+        }
+
+        destLayout->refcount++;
+    }
+
+    size_t deepBytecountConcrete(instance_ptr instance, std::unordered_set<void*>& alreadyVisited, std::set<Slab*>* outSlabs) {
+        layout_ptr& self_layout = *(layout_ptr*)instance;
+
+        if (!self_layout) {
+            return 0;
+        }
+
+        if (alreadyVisited.find((void*)self_layout) != alreadyVisited.end()) {
+            return 0;
+        }
+
+        alreadyVisited.insert((void*)self_layout);
+
+        if (outSlabs && Slab::slabForAlloc(self_layout)) {
+            outSlabs->insert(Slab::slabForAlloc(self_layout));
+            return 0;
+        }
+
+        size_t reserveCount = self_layout->count;
+
+        // always allocate something for a list
+        if (reserveCount == 0 && !m_is_tuple) {
+            reserveCount = 1;
+        }
+
+        size_t res = bytesRequiredForAllocation(sizeof(layout));
+
+        if (reserveCount) {
+            res += bytesRequiredForAllocation(reserveCount * getEltType()->bytecount());
+        }
+
+        if (!getEltType()->isPOD()) {
+            for (long k = 0; k < self_layout->count; k++) {
+                res += m_element_type->deepBytecount(eltPtr(instance, k), alreadyVisited, outSlabs);
+            }
+        }
+
+        return res;
+    }
+
     void repr(instance_ptr self, ReprAccumulator& stream, bool isStr);
 
     typed_python_hash_type hash(instance_ptr left);
@@ -110,13 +206,13 @@ public:
             return;
         }
 
-        self = (layout*)malloc(sizeof(layout));
+        self = (layout*)tp_malloc(sizeof(layout));
 
         self->count = count;
         self->refcount = 1;
         self->reserved = std::max<int32_t>(1, count);
         self->hash_cache = -1;
-        self->data = (uint8_t*)malloc(getEltType()->bytecount() * self->reserved);
+        self->data = (uint8_t*)tp_malloc(getEltType()->bytecount() * self->reserved);
 
         for (int64_t k = 0; k < count; k++) {
             try {
@@ -125,8 +221,8 @@ public:
                 for (long k2 = k-1; k2 >= 0; k2--) {
                     m_element_type->destroy(eltPtr(self,k2));
                 }
-                free(self->data);
-                free(self);
+                tp_free(self->data);
+                tp_free(self);
                 throw;
             }
         }
@@ -137,21 +233,21 @@ public:
     void constructorUnbounded(instance_ptr selfPtr, const sub_constructor& allocator) {
         layout_ptr& self = *(layout_ptr*)selfPtr;
 
-        self = (layout*)malloc(sizeof(layout));
+        self = (layout*)tp_malloc(sizeof(layout));
 
         self->count = 0;
         self->refcount = 1;
         self->reserved = 1;
         self->hash_cache = -1;
-        self->data = (uint8_t*)malloc(getEltType()->bytecount() * self->reserved);
+        self->data = (uint8_t*)tp_malloc(getEltType()->bytecount() * self->reserved);
 
         while(true) {
             try {
                 if (!allocator(eltPtr(self, self->count), self->count)) {
                     if (m_is_tuple && self->count == 0) {
                         //tuples need to be the nullptr
-                        free(self->data);
-                        free(self);
+                        tp_free(self->data);
+                        tp_free(self);
                         self = nullptr;
                     }
                     return;
@@ -165,8 +261,8 @@ public:
                 for (long k2 = (long)self->count-1; k2 >= 0; k2--) {
                     m_element_type->destroy(eltPtr(self,k2));
                 }
-                free(self->data);
-                free(self);
+                tp_free(self->data);
+                tp_free(self);
                 throw;
             }
         }
