@@ -59,6 +59,7 @@ class Float64;
 class StringType;
 class BytesType;
 class OneOfType;
+class SubclassOfType;
 class Value;
 class TupleOfType;
 class PointerTo;
@@ -141,6 +142,7 @@ public:
         catTypedCell = 34,
         catPyCell = 35,
         catRefTo = 36,
+        catSubclassOf = 38, // holds a Type* which is a subclass of a given set of Type instances
     };
 
     bool isTuple() const {
@@ -223,6 +225,10 @@ public:
         return m_typeCategory == catOneOf;
     }
 
+    bool isSubclassOf() const {
+        return m_typeCategory == catSubclassOf;
+    }
+
     bool isValue() const {
         return m_typeCategory == catValue;
     }
@@ -285,6 +291,16 @@ public:
         return m_name;
     }
 
+    std::string nameWithModule() {
+        return this->check([&](auto& subtype) {
+            return subtype.nameWithModuleConcrete();
+        });
+    }
+
+    std::string nameWithModuleConcrete() {
+        return name();
+    }
+
     const char* doc() const {
         return m_doc;
     }
@@ -433,6 +449,7 @@ public:
         if (category == Type::TypeCategory::catPythonObjectOfType) { return "PythonObjectOfType"; }
         if (category == Type::TypeCategory::catPyCell) { return "PyCell"; }
         if (category == Type::TypeCategory::catTypedCell) { return "TypedCell"; }
+        if (category == Type::TypeCategory::catSubclassOf) { return "SubclassOf"; }
 
         return "Unknown";
     }
@@ -516,6 +533,8 @@ public:
                 return f(*(PyCellType*)this);
             case catTypedCell:
                 return f(*(TypedCellType*)this);
+            case catSubclassOf:
+                return f(*(SubclassOfType*)this);
             default:
                 throw std::runtime_error("Invalid type found");
         }
@@ -752,14 +771,8 @@ public:
         m_recursive_forward_index = index;
     }
 
-    // can we construct an instance of 'this' from an instance of 'otherType'
-    // if 'True' then we can always do this. if False, then never. If Maybe, then
-    // we cannot make any assumptions..
-    Maybe canConstructFrom(Type* otherType, bool isExplicit);
-
-    Maybe canConstructFromConcrete(Type* otherType, bool isExplicit) {
-        return Maybe::Maybe;
-    }
+    // are we guaranteed we can convert to this other type at the 'Signature' level
+    bool canConvertToTrivially(Type* otherType);
 
     /*****
         determine the full group of types that this type is a part of as far
@@ -932,10 +945,6 @@ protected:
 
     std::map<Type*, BinaryCompatibilityCategory> mIsBinaryCompatible;
 
-    std::map<Type*, Maybe> mCanConvert;
-
-    std::set<Type*> mCanConvertOnStack;
-
     // a set of forward types that we need to be resolved before we
     // could be resolved
     std::set<Forward*> m_referenced_forwards;
