@@ -113,6 +113,7 @@ extern "C" {
         if (Type::typesEquivalent(derived, super)) {
             return true;
         }
+
         return derived->isSubclassOf(super);
     }
 
@@ -203,6 +204,30 @@ extern "C" {
         throw PythonExceptionSet();
     }
 
+    ClassDispatchTable* computeTypeClassDispatchTable(Type* concreteType, Type* knownAsType) {
+        if (concreteType->isClass()) {
+            concreteType = ((Class*)concreteType)->getHeldClass();
+        }
+
+        if (knownAsType->isClass()) {
+            knownAsType = ((Class*)knownAsType)->getHeldClass();
+        }
+
+        HeldClass* concreteCls = (HeldClass*)concreteType;
+        HeldClass* knownAsCls = (HeldClass*)knownAsType;
+
+        VTable* table = concreteCls->getVTable();
+        ClassDispatchTable* dispatchTable = table->mDispatchTables;
+        while (true) {
+            if (dispatchTable->getInterfaceClass() == knownAsCls) {
+                return dispatchTable;
+            }
+
+            dispatchTable++;
+        }
+    }
+
+
     // START math functions
     // parameters are checked before calling these functions
 
@@ -2384,7 +2409,7 @@ extern "C" {
             ret = PyFloat_AsDouble(retObj);
         }
         else if (PyLong_Check(retObj)) {
-             ret = (double)PyLong_AsLong(retObj);
+             ret = PyLong_AsDouble(retObj);
         }
         else {
             PyObjectHolder floatObj(PyObject_CallMethod(retObj, "__float__", NULL));
@@ -2421,7 +2446,7 @@ extern "C" {
             ret = PyFloat_AsDouble(retObj);
         }
         else if (PyLong_Check(retObj)) {
-            ret = (double)PyLong_AsLong(retObj);
+            ret = PyLong_AsDouble(retObj);
         }
         else {
             PyObjectHolder floatObj(PyObject_CallMethod(retObj, "__float__", NULL));
@@ -2472,7 +2497,7 @@ extern "C" {
             ret = PyFloat_AsDouble(retObj);
         }
         else if (PyLong_Check(retObj)) {
-             ret = (double)PyLong_AsLong(retObj);
+             ret = PyLong_AsDouble(retObj);
         }
         else {
             PyObjectHolder floatObj(PyObject_CallMethod(retObj, "__float__", NULL));
