@@ -66,7 +66,6 @@ class ExpressionConversionContext:
     evaluation, and provides convenience methods to allow expression generators to stash
     compound expressions and get back simple variable references.
     """
-
     def __init__(self, functionContext, variableStates: FunctionStackState):
         self.functionContext = functionContext
         self.intermediates = []
@@ -117,6 +116,9 @@ class ExpressionConversionContext:
         ).load()
 
     def allocateClassMethodDispatchSlot(self, clsType, methodName, retType, argTupleType, kwargTupleType):
+        # the first argument indicates whether this is an instance or type-level dispatch
+        assert argTupleType.ElementTypes[0].Value in ('type', 'instance')
+
         identHash = self.converter.hashObjectToIdentity(
             (clsType, methodName, retType, argTupleType, kwargTupleType)
         )
@@ -670,6 +672,9 @@ class ExpressionConversionContext:
 
             def __exit__(scope, t, v, traceback):
                 if t is None:
+                    if not targets:
+                        return
+
                     expr = results.get(targets[-1], native_ast.nullExpr)
 
                     for t in reversed(targets[:-1]):
@@ -1170,6 +1175,13 @@ class ExpressionConversionContext:
             for path in overload.closureVarLookups.values()
         ]
 
+        if returnTypeOverload is not None:
+            returnType = returnTypeOverload
+        elif overload.signatureFunction:
+            raise Exception("At this point, the signature should be fully known")
+        else:
+            returnType = typeWrapper(overload.returnType) if overload.returnType is not None else None
+
         call_target = self.functionContext.converter.convert(
             overload.name,
             overload.functionCode,
@@ -1179,9 +1191,7 @@ class ExpressionConversionContext:
             list(overload.closureVarLookups),
             [a.expr_type for a in closureArgs]
             + [a.expr_type for a in concreteArgs],
-            returnTypeOverload if returnTypeOverload is not None else
-            typeWrapper(overload.returnType) if overload.returnType is not None else
-            None
+            returnType
         )
 
         if call_target is None:
@@ -1630,13 +1640,7 @@ class ExpressionConversionContext:
             if val is None:
                 return None
 
-            if ast.slice.matches.Index:
-                index = self.convert_expression_ast(ast.slice.value)
-                if index is None:
-                    return None
-
-                return val.convert_getitem(index)
-            elif ast.slice.matches.Slice:
+            if ast.slice.matches.Slice:
                 if ast.slice.lower is None:
                     lower = None
                 else:
@@ -1659,16 +1663,10 @@ class ExpressionConversionContext:
                         return None
 
                 return val.convert_getslice(lower, upper, step)
-            elif ast.slice.matches.ExtSlice:
+            elif ast.slice.matches.Tuple:
                 args = []
-                for dim in ast.slice.dims:
-                    if dim.matches.Index:
-                        index = self.convert_expression_ast(ast.slice.value)
-                        if index is None:
-                            return None
-
-                        args.append(index)
-                    elif dim.matches.Slice:
+                for dim in ast.slice.elts:
+                    if dim.matches.Slice:
                         if dim.lower is None:
                             lower = self.constant(None)
                         else:
@@ -1693,6 +1691,12 @@ class ExpressionConversionContext:
                         args.append(
                             SliceWrapper().convert_call(self, None, [lower, upper, step], {})
                         )
+                    else:
+                        index = self.convert_expression_ast(ast.slice)
+                        if index is None:
+                            return None
+
+                        args.append(index)
 
                 tupType = Tuple(*[x.expr_type.typeRepresentation for x in args])
 
@@ -1710,7 +1714,11 @@ class ExpressionConversionContext:
 
                 return val.convert_getitem(index)
             else:
-                assert False, type(ast.slice)
+                index = self.convert_expression_ast(ast.slice)
+                if index is None:
+                    return None
+
+                return val.convert_getitem(index)
 
         if ast.matches.Call:
             lhs = self.convert_expression_ast(ast.func)
