@@ -158,7 +158,7 @@ def constant_to_typed_llvm_value(module, builder, c):
         return TypedLLVMValue(None, native_ast.Type.Void())
 
     if c.matches.Array:
-        vals = [constant_to_typed_llvm_value(module, builder, t) for _, t in c.values]
+        vals = [constant_to_typed_llvm_value(module, builder, t) for t in c.values]
 
         t = llvmlite.ir.ArrayType(type_to_llvm_type(c.value_type.element_type), c.value_type.count)
 
@@ -1309,7 +1309,7 @@ class FunctionConverter:
                 else:
                     llvm_call_result = self.builder.call(func.llvm_value, [a.llvm_value for a in args])
             except Exception:
-                print("failing while calling ", target.name)
+                print("failing while calling ", target)
                 for a in args:
                     print("\t", a.llvm_value, a.native_type)
                 raise
