@@ -14,6 +14,7 @@
 
 import typed_python.compiler.native_ast as native_ast
 from typed_python.compiler.expression_conversion_context import ExpressionConversionContext
+from typed_python.compiler.function_metadata import FunctionMetadata
 
 
 class NativeFunctionConversionContext:
@@ -29,6 +30,7 @@ class NativeFunctionConversionContext:
         self._output_type = output_type
         self._generatingFunction = generatingFunction
         self._identity = identity
+        self.functionMetadata = FunctionMetadata()
 
     def getInputTypes(self):
         return self._input_types
@@ -37,6 +39,9 @@ class NativeFunctionConversionContext:
         """If the output type is known ahead, then that type (as a wrapper). Else, None"""
         return self._output_type
 
+    def __str__(self):
+        return f"NativeFunctionConversionContext({self._identity}, {self._generatingFunction})"
+
     @property
     def identity(self):
         return self._identity
@@ -65,6 +70,8 @@ class NativeFunctionConversionContext:
 
     def getFunction(self):
         self.varnames = 0
+        # reset the constant return value.
+        self.functionMetadata = FunctionMetadata()
 
         try:
             subcontext = ExpressionConversionContext(self, None)
