@@ -29,7 +29,9 @@ from typed_python.compiler.type_wrappers.class_wrapper import ClassWrapper
 from typed_python.compiler.python_object_representation import typedPythonTypeToTypeWrapper
 from typed_python.compiler.function_conversion_context import FunctionConversionContext, FunctionOutput, FunctionYield
 from typed_python.compiler.native_function_conversion_context import NativeFunctionConversionContext
-from typed_python.compiler.type_wrappers.python_typed_function_wrapper import PythonTypedFunctionWrapper
+from typed_python.compiler.type_wrappers.python_typed_function_wrapper import (
+    PythonTypedFunctionWrapper, CannotBeDetermined, NoReturnTypeSpecified
+)
 from typed_python.compiler.typed_call_target import TypedCallTarget
 
 typeWrapper = lambda t: typed_python.compiler.python_object_representation.typedPythonTypeToTypeWrapper(t)
@@ -149,6 +151,9 @@ class PythonToNativeConverter:
 
         self._dependencies = FunctionDependencyGraph()
 
+    def isCurrentlyConverting(self):
+        return len(self._inflight_function_conversions) > 0
+
     def getDefinitionCount(self):
         return len(self._definitions)
 
@@ -566,7 +571,7 @@ class PythonToNativeConverter:
         # we are compiling the function 'name' in 'implementingClass' to be installed when
         # viewing an instance of 'implementingClass' as 'interfaceClass' that's function
         # 'name' called with signature '(*argTypeTuple, **kwargTypeTuple) -> retType'
-        typedCallTarget = ClassWrapper.compileMethodInstantiation(
+        typedCallTarget = ClassWrapper.compileVirtualMethodInstantiation(
             self,
             interfaceClass,
             implementingClass,
@@ -631,6 +636,18 @@ class PythonToNativeConverter:
 
         realizedInputWrappers.extend(inputWrappers)
 
+        returnType = PythonTypedFunctionWrapper.computeFunctionOverloadReturnType(
+            overload,
+            inputWrappers,
+            {}
+        )
+
+        if returnType is CannotBeDetermined:
+            returnType = object
+
+        if returnType is NoReturnTypeSpecified:
+            returnType = None
+
         return self.convert(
             overload.name,
             overload.functionCode,
@@ -639,11 +656,11 @@ class PythonToNativeConverter:
             overload.funcGlobalsInCells,
             list(overload.closureVarLookups),
             realizedInputWrappers,
-            overload.returnType,
+            returnType,
             assertIsRoot=assertIsRoot
         )
 
-    def hashObjectToIdentity(self, hashable):
+    def hashObjectToIdentity(self, hashable, isModuleVal=False):
         if isinstance(hashable, Hash):
             return hashable
 
@@ -656,17 +673,14 @@ class PythonToNativeConverter:
         if hashable is None:
             return Hash.from_integer(1) + Hash.from_integer(0)
 
+        if isinstance(hashable, (dict, list)) and isModuleVal:
+            # don't look into dicts and lists at module level
+            return Hash.from_integer(2)
+
         if isinstance(hashable, (tuple, list)):
             res = Hash.from_integer(len(hashable))
             for t in hashable:
-                res += self.hashObjectToIdentity(t)
-            return res
-
-        if isinstance(hashable, dict):
-            res = Hash.from_integer(len(hashable))
-            for t in sorted(hashable):
-                res += self.hashObjectToIdentity(t)
-                res += self.hashObjectToIdentity(hashable[t])
+                res += self.hashObjectToIdentity(t, isModuleVal)
             return res
 
         if isinstance(hashable, Wrapper):
@@ -699,12 +713,12 @@ class PythonToNativeConverter:
         item = funcGlobals[dotSeq[0]]
 
         if not isinstance(item, ModuleType) or len(dotSeq) == 1:
-            return Hash.from_string(dotSeq[0]) + self.hashObjectToIdentity(item)
+            return Hash.from_string(dotSeq[0]) + self.hashObjectToIdentity(item, True)
 
         if not hasattr(item, dotSeq[1]):
             return Hash.from_integer(0)
 
-        return Hash.from_string(dotSeq[0] + "." + dotSeq[1]) + self.hashObjectToIdentity(getattr(item, dotSeq[1]))
+        return Hash.from_string(dotSeq[0] + "." + dotSeq[1]) + self.hashObjectToIdentity(getattr(item, dotSeq[1]), True)
 
     def convert(
         self,
