@@ -35,10 +35,14 @@ from typed_python import (
     Value,
     pointerTo,
     refTo,
-    Forward
+    Forward,
+    TypeFunction,
+    typeKnownToCompiler,
+    SubclassOf
 )
 import typed_python._types as _types
 from typed_python.compiler.runtime import Entrypoint, Runtime, CountCompilationsVisitor
+from typed_python.compiler.type_wrappers.class_wrapper import classCouldBeInstanceOf
 
 
 def resultType(f, **kwargs):
@@ -420,25 +424,6 @@ class TestClassCompilationCompilation(unittest.TestCase):
 
         self.assertEqual(_types.refcount(aListOfInt), 1)
 
-    def test_class_inheritance_method_signatures(self):
-        class Base(Class):
-            def f(self) -> int:
-                return 0
-
-        # if you override a method you can't change its output type.
-        with self.assertRaisesRegex(Exception, "Overloads of 'f' don't have the same return type"):
-
-            class BadChild(Base):
-                def f(self) -> float:
-                    pass
-
-        # but if you have a non-overlapping signature it's OK because
-        # the compiler wouldn't have a problem figuring out which one
-        # to dispatch to.
-        class GoodChild(Base):
-            def f(self, x) -> float:
-                pass
-
     def test_dispatch_with_multiple_overloads(self):
         # check that we can dispatch appropriately
         class TestClass(Class):
@@ -2383,6 +2368,8 @@ class TestClassCompilationCompilation(unittest.TestCase):
             isE = 0
             isInt = 0
 
+            assert typeKnownToCompiler(c) is C
+
             for _ in range(times):
                 if isinstance(c, C):
                     isC += 1
@@ -2395,14 +2382,19 @@ class TestClassCompilationCompilation(unittest.TestCase):
 
             return (isC, isD, isE, isInt)
 
-        countIt(1, D())
-        t0 = time.time()
-        countIt(1000000, D())
-        print(time.time() - t0, " to do 1mm")
+        assert countIt(1, D()) == (1, 1, 0, 0)
+        assert countIt(1, C()) == (1, 0, 0, 0)
+        assert countIt(1, E()) == (1, 0, 1, 0)
 
-        # I get 0.002 on my machine. This should be very fast
-        # because either the compiler can figure it out
-        assert time.time() - t0 < 0.01
+        t0 = time.time()
+        assert countIt(1000000, D())[1] == 1000000
+        elapsed = time.time() - t0
+        print(elapsed, " to do 1mm")
+
+        # I get 1e-5 on my machine. This code doesn't have to hit
+        # any functions to determine this because final classes have
+        # an exactly known type.
+        assert elapsed < 0.01
 
     def test_unresolved_forwards_in_class_type_signatures(self):
         UnresolvedForward = Forward("UnresolvedForward")
@@ -2431,3 +2423,422 @@ class TestClassCompilationCompilation(unittest.TestCase):
 
         with self.assertRaisesRegex(TypeError, "has unresolved forwards"):
             tryToCallF()
+
+    def test_could_be_instance_of(self):
+        class A(Class):
+            pass
+
+        class B(Class):
+            pass
+
+        class C(A, B):
+            pass
+
+        # instances of A can be B and vice versa
+        assert classCouldBeInstanceOf(A, B) == "Maybe"
+        assert classCouldBeInstanceOf(B, A) == "Maybe"
+
+        # instances of A and B can both be 'C'
+        assert classCouldBeInstanceOf(A, C) == "Maybe"
+        assert classCouldBeInstanceOf(B, C) == "Maybe"
+
+        # instances of C are definitely A and B
+        assert classCouldBeInstanceOf(C, A) == True  # noqa
+        assert classCouldBeInstanceOf(C, B) == True  # noqa
+
+        class AFinal(A, Final):
+            pass
+
+        class BFinal(B, Final):
+            pass
+
+        # instances of B cannot be AFinal, because we know exactly what base classes could
+        # back a 'B'
+        assert classCouldBeInstanceOf(A, BFinal) == False  # noqa
+
+        class AWithMembers(A):
+            x = Member(int)
+
+        class BWithMembers(B):
+            x = Member(int)
+
+        # an instance of A could be a B with members, because its possible for
+        # a subclass of BWithMembers to descend from A
+        assert classCouldBeInstanceOf(A, BWithMembers) == "Maybe"
+
+        # but if its an AWithMembers, then its not possible to create that subclass
+        assert classCouldBeInstanceOf(AWithMembers, BWithMembers) == False  # noqa
+
+        class AWithType(A):
+            X = int
+
+        class BWithType(B):
+            X = float
+
+        # an instance of A could be a BWithType
+        assert classCouldBeInstanceOf(A, BWithType) == "Maybe"
+
+        # but if its an AWithType, then its not possible to create that subclass
+        assert classCouldBeInstanceOf(AWithType, BWithType) == False  # noqa
+
+        # its possible to descend from two separate type functions that don't declare
+        # any classmembers
+        @TypeFunction
+        def TF1(X):
+            class TF1_(TF1):
+                __name__ = f"TF2({X})"
+
+                def f(self) -> X:
+                    return X()
+
+            return TF1_
+
+        @TypeFunction
+        def TF2(X):
+            class TF2_(TF2):
+                __name__ = f"TF2({X})"
+
+                def g(self) -> X:
+                    return X()
+
+            return TF2_
+
+        assert classCouldBeInstanceOf(TF1(int), TF2(float))
+
+        class ActualSubclass(TF1(int), TF2(float)):
+            pass
+
+    def test_super(self):
+        class B(Class):
+            def f(self, x):
+                return x
+
+        class C(B):
+            def f(self, x):
+                return super().f(x) + 1
+
+        @Entrypoint
+        def callIt(c, x):
+            return c.f(x)
+
+        assert callIt(C(), 10) == 11
+
+    def test_super_with_diamond(self):
+        class B(Class):
+            def f(self, x: ListOf(str)):
+                x.append('B')
+
+        class C(B):
+            def f(self, x):
+                x.append('C')
+                super().f(x)
+
+        class D(B):
+            def f(self, x):
+                x.append('D')
+                super().f(x)
+
+        class E(C, D):
+            def f(self, x):
+                x.append('E')
+                super().f(x)
+
+        lst = ListOf(str)()
+        E().f(lst)
+
+        assert lst == ['E', 'C', 'D', 'B']
+
+        @Entrypoint
+        def callIt(c, x):
+            return c.f(x)
+
+        lst.resize(0)
+        callIt(E(), lst)
+
+        assert lst == ['E', 'C', 'D', 'B']
+
+    def test_super_with_no_superclass(self):
+        class B(Class):
+            def f(self):
+                return super().f()
+
+        @Entrypoint
+        def callIt(c):
+            return c.f()
+
+        with self.assertRaisesRegex(AttributeError, r"'super' object has no attribute 'f'"):
+            B().f()
+
+        with self.assertRaisesRegex(AttributeError, r"'super' object has no attribute 'f'"):
+            callIt(B())
+
+    def test_super_in_init(self):
+        class B(Class):
+            x = Member(int)
+
+            def __init__(self, x):
+                self.x = x
+
+        class C(B):
+            y = Member(int)
+
+            def __init__(self, x, y):
+                super().__init__(x)
+                self.y = y
+
+        def check():
+            c = C(10, 11)
+            assert c.x == 10
+            assert c.y == 11
+
+        check()
+        Entrypoint(check)()
+
+    def test_compile_classmethods(self):
+        class Normal:
+            @classmethod
+            def f(cls):
+                return cls
+
+        assert Normal.f() is Normal
+        assert Normal().f() is Normal
+
+        class A(Class):
+            @classmethod
+            def f(cls):
+                return cls
+
+        def checkIt():
+            assert A.f() is A
+            assert A().f() is A
+
+        checkIt()
+        Entrypoint(checkIt)()
+
+    def test_compile_classmethod_on_subclass_multi(self):
+        class B(Class):
+            @classmethod
+            def f(cls, x: int) -> int:
+                return 1
+
+            @classmethod
+            def f(cls, x: float) -> float:  # noqa
+                return 1.5
+
+        class C(B):
+            @classmethod
+            def f(cls, x: int) -> int:
+                return 2
+
+        @Entrypoint
+        def callIt(b: B, x: OneOf(int, float)):
+            return b.f(x)
+
+        assert callIt(B(), 1) == 1
+        assert callIt(B(), 1.0) == 1.5
+        assert callIt(C(), 1) == 2
+        assert callIt(C(), 1.0) == 1.5
+
+    def test_compile_staticmethod_on_subclass_multi(self):
+        class B(Class):
+            @staticmethod
+            def f(x: int) -> int:
+                return 1
+
+            @staticmethod
+            def f(x: float) -> float:  # noqa
+                return 1.5
+
+        class C(B):
+            @staticmethod
+            def f(x: int) -> int:
+                return 2
+
+        @Entrypoint
+        def callIt(b: B, x: OneOf(int, float)):
+            return b.f(x)
+
+        assert callIt(B(), 1) == 1
+        assert callIt(B(), 1.0) == 1.5
+        assert callIt(C(), 1) == 2
+        assert callIt(C(), 1.0) == 1.5
+
+    def test_compile_classmethod_on_subclass(self):
+        class BPy(object):
+            @classmethod
+            def f(cls, x) -> int:
+                return 1
+
+        class CPy(BPy):
+            @classmethod
+            def f(cls, x) -> int:
+                return 2
+
+        class DPy(CPy):
+            @classmethod
+            def f(cls, x) -> int:
+                return 3
+
+        def callItPy(b, times):
+            x = 0
+            for i in range(times):
+                x += b.f(i)
+            return x
+
+        t0 = time.time()
+        callItPy(BPy(), 1000000)
+        callItPy(CPy(), 1000000)
+        pyTime = time.time() - t0
+
+        class B(Class):
+            @classmethod
+            def f(cls, x) -> int:
+                return 1
+
+        class C(B):
+            @classmethod
+            def f(cls, x) -> int:
+                return 2
+
+        class D(C, Final):
+            @classmethod
+            def f(cls, x) -> int:
+                return 3
+
+        def callItAs(T):
+            @Entrypoint
+            def callIt(b: T, times: int):
+                x = 0
+                for i in range(times):
+                    x += b.f(i)
+                return x
+
+            return callIt
+
+        def callItAsWithType(T):
+            @Entrypoint
+            def callIt(b: SubclassOf(T), times: int):
+                x = 0
+                for i in range(times):
+                    x += b.f(i)
+                return x
+
+            return callIt
+
+        callItAs(B)(B(), 1)
+        callItAs(B)(C(), 1)
+        callItAsWithType(B)(B, 1)
+        callItAsWithType(B)(C, 1)
+
+        t0 = time.time()
+        assert callItAs(B)(B(), 1000000) == 1000000
+        assert callItAs(B)(C(), 1000000) == 2000000
+        compiledTime = time.time() - t0
+
+        t0 = time.time()
+        assert callItAsWithType(B)(B, 1000000) == 1000000
+        assert callItAsWithType(B)(C, 1000000) == 2000000
+        compiledTypeTime = time.time() - t0
+
+        speedup = pyTime / compiledTime
+        speedupType = pyTime / compiledTypeTime
+
+        print("compiled virtual dispatch on instances ", speedup, "times faster")
+        print("compiled virtual dispatch on type ", speedupType, "times faster")
+
+    def test_compile_staticmethod_on_subclass(self):
+        class B(Class):
+            @staticmethod
+            def f(x) -> int:
+                return 1
+
+        class C(B):
+            @staticmethod
+            def f(x) -> int:
+                return 2
+
+        class D(C, Final):
+            @staticmethod
+            def f(x) -> int:
+                return 3
+
+        def callItAs(T):
+            @Entrypoint
+            def callIt(b: T, times: int):
+                x = 0
+                for i in range(times):
+                    x += b.f(i)
+                return x
+
+            return callIt
+
+        assert B().f(10) == 1
+
+        assert callItAs(B)(B(), 1000000) == 1000000
+        assert callItAs(B)(C(), 1000000) == 2000000
+
+    def test_class_dispatch_on_type(self):
+        class B(Class):
+            @classmethod
+            def f(cls) -> int:
+                return 0
+
+        @Entrypoint
+        def call(T: SubclassOf(B)):
+            return T.f()
+
+        assert call.resultTypeFor(SubclassOf(B)).typeRepresentation is int
+        assert call(B) == 0
+
+    def test_interpreter_can_see_class_members(self):
+        class B(Class):
+            X = int
+
+        @Entrypoint
+        def callType(T: SubclassOf(B)):
+            return T.X
+
+        @Entrypoint
+        def callInst(T: B):
+            return T.X
+
+        assert callType.resultTypeFor(SubclassOf(B)).typeRepresentation.Value is int
+        assert callInst.resultTypeFor(B).typeRepresentation.Value is int
+
+    def test_class_init_is_virtual(self):
+        B = Forward("B")
+
+        @B.define
+        class B(Class):
+            x = Member(int)
+
+            def __init__(self, x):
+                self.x = x
+
+            def f(self) -> int:
+                return self.x
+
+            @classmethod
+            def makeOne(cls, x) -> B:
+                return cls(x)
+
+        class C(B):
+            def __init__(self, x):
+                super().__init__(x + 10)
+
+            def f(self):
+                return self.x + 100
+
+        @Entrypoint
+        def constructOne(T: SubclassOf(B), x, times):
+            res = 0
+            for i in range(times):
+                res += T.makeOne(x).f()
+            return res
+
+        assert constructOne(C, 0, 1) == C(0).f()
+        assert constructOne.resultTypeFor(SubclassOf(B), int, int).typeRepresentation is int
+
+        t0 = time.time()
+        assert constructOne(C, 0, 1000000) == C(0).f() * 1000000
+        print(time.time() - t0)
