@@ -19,8 +19,9 @@ from typed_python import (
     localVariableTypesKnownToCompiler
 )
 from typed_python.compiler.runtime import Entrypoint
+from typed_python.compiler.runtime import Runtime
 import unittest
-
+import time
 
 A = Alternative(
     "A",
@@ -293,7 +294,7 @@ class TestTypeInference(unittest.TestCase):
 
             return typeKnownToCompiler(y)
 
-        assert returnTypeOfArgument(10) is OneOf(int, float)
+        assert returnTypeOfArgument(10) is OneOf(float, int)
 
     def test_compiler_knows_that_isinstance_constrains_types(self):
         @Entrypoint
@@ -322,4 +323,61 @@ class TestTypeInference(unittest.TestCase):
             return localVariableTypesKnownToCompiler()
 
         res = localVariableTypes(10)
-        assert res == dict(x=int, y=OneOf(int, float))
+        assert res == dict(x=int, y=OneOf(float, int))
+
+    def test_type_inference_perf(self):
+        t0 = time.time()
+        f = Function(lambda o: o + 1)
+
+        Runtime.singleton().resultTypeForCall(f, [int], {})
+
+        for _ in range(10000):
+            Runtime.singleton().resultTypeForCall(f, [int], {})
+
+        # I get around 0.03 on my desktop with the caching, and .7 without it
+        assert time.time() - t0 < .4
+
+    def test_type_inference_with_exceptions(self):
+        # check that if we catch an arbitrary exception, then we can't really
+        # know what the type of a variable coming out of the exception block.
+        # this is a little crude because obviously we can tell below because we
+        # know we are not throwing exceptions, but we want to ensure that we don't
+        # mistakenly assume it's 'str'.
+        @Entrypoint
+        def check():
+            try:
+                y = 10
+                y = "hi"  # noqa
+            except Exception:
+                pass
+
+            return localVariableTypesKnownToCompiler()
+
+        assert check() == dict(y=OneOf(int, str))
+
+        # but if we return from the handler directly, then the only way to get to the
+        # code after the except is if everything passed.
+        @Entrypoint
+        def check2():
+            try:
+                y = 10
+                y = "hi"  # noqa
+            except Exception:
+                return
+
+            return localVariableTypesKnownToCompiler()
+
+        assert check2()['y'] == str
+
+        # check that the 'finally' block knows it could be the merge
+        @Entrypoint
+        def check3():
+            try:
+                y = 10
+                y = "hi"  # noqa
+            except Exception:
+                y = 12.5  # noqa
+            finally:
+                return localVariableTypesKnownToCompiler()
+
+        assert check3()['y'] == OneOf(float, str)
