@@ -55,19 +55,10 @@ class IsinstanceWrapper(Wrapper):
     @Wrapper.unwrapOneOfAndValue
     def convert_call(self, context, expr, args, kwargs):
         if len(args) == 2 and not kwargs:
-            if args[1].expr_type.is_py_type_object_wrapper:
-                if isinstance(args[0].expr_type.typeRepresentation, type):
-                    if issubclass(
-                        args[0].expr_type.typeRepresentation,
-                        args[1].expr_type.typeRepresentation.Value
-                    ):
-                        return context.constant(True)
-                    if cannotBeSubclass(
-                        args[0].expr_type.typeRepresentation,
-                        args[1].expr_type.typeRepresentation.Value
-                    ):
-                        return context.constant(False)
+            instance = args[0]
+            typeObj = args[1]
+            instanceType = instance.convert_typeof()
 
-            return args[1].expr_type.convert_issubclass(context, args[1], args[0].convert_typeof(), False)
+            return instanceType.convert_issubclass(typeObj, False)
 
         return super().convert_call(context, expr, args, kwargs)
