@@ -266,6 +266,8 @@ class ClassMetaclass(type):
         if not bases:
             return type.__new__(cls, name, bases, namespace.ns, **kwds)
 
+        classCell = namespace.ns.get('__classcell__')
+
         members = []
         isFinal = Final in bases
 
@@ -273,16 +275,40 @@ class ClassMetaclass(type):
 
         memberFunctions = {}
         staticFunctions = {}
-        classMembers = []
+        classMembers = {}
+        classMethods = {}
         properties = {}
 
+        if "__name__" in kwds:
+            name = kwds["__name__"]
+
         for eltName, elt in namespace.order:
-            if isinstance(elt, Member):
+            if eltName == '__classcell__':
+                pass
+            elif eltName == '__name__':
+                name = elt
+            elif isinstance(elt, Member):
                 members.append((eltName, elt._type, elt._default_value, elt._nonempty))
-                classMembers.append((eltName, elt))
+                classMembers[eltName] = elt
             elif isinstance(elt, property):
                 properties[eltName] = makeFunctionType(eltName, elt.fget, assumeClosuresGlobal=True)
+            elif isinstance(elt, classmethod):
+                if eltName in staticFunctions:
+                    raise TypeError(f"method {eltName} can't be both a staticmethod and a classmethod")
+
+                if eltName not in classMethods:
+                    classMethods[eltName] = makeFunctionType(
+                        eltName, elt.__func__, assumeClosuresGlobal=True
+                    )
+                else:
+                    classMethods[eltName] = typed_python._types.Function(
+                        classMethods[eltName],
+                        makeFunctionType(eltName, elt.__func__, assumeClosuresGlobal=True),
+                    )
             elif isinstance(elt, staticmethod):
+                if eltName in classMethods:
+                    raise TypeError(f"method {eltName} can't be both a staticmethod and a classmethod")
+
                 if eltName not in staticFunctions:
                     staticFunctions[eltName] = makeFunctionType(
                         eltName, elt.__func__, assumeClosuresGlobal=True
@@ -298,7 +324,9 @@ class ClassMetaclass(type):
                 or isinstance(elt, type)
                 and issubclass(elt, typed_python._types.Function)
             ):
-                if eltName not in memberFunctions:
+                if eltName == '__typed_python_template__':
+                    classMembers[eltName] = elt
+                elif eltName not in memberFunctions:
                     memberFunctions[eltName] = makeFunctionType(
                         eltName, elt, classname=name, assumeClosuresGlobal=True
                     )
@@ -308,17 +336,9 @@ class ClassMetaclass(type):
                         makeFunctionType(eltName, elt, classname=name, assumeClosuresGlobal=True),
                     )
             else:
-                classMembers.append((eltName, elt))
+                classMembers[eltName] = elt
 
-                if eltName == "__module__":
-                    # __module__ gets swallowed by the base Class instance, so we have to
-                    # communicate the variable by sticking the 'typed_python' in it
-                    classMembers.append(("__typed_python_module__", elt))
-
-        if "__name__" in kwds:
-            name = kwds["__name__"]
-
-        return typed_python._types.Class(
+        res = typed_python._types.Class(
             name,
             tuple(bases),
             isFinal,
@@ -326,10 +346,19 @@ class ClassMetaclass(type):
             tuple(memberFunctions.items()),
             tuple(staticFunctions.items()),
             tuple(properties.items()),
-            tuple(classMembers),
+            tuple(classMembers.items()),
+            tuple(classMethods.items()),
         )
 
+        if classCell is not None:
+            typed_python._types.setPyCellContents(classCell, res)
+
+        return res
+
     def __subclasscheck__(cls, subcls):
+        if cls is subcls:
+            return True
+
         if getattr(subcls, "__typed_python_category__", None) != "Class":
             return False
 
@@ -339,6 +368,9 @@ class ClassMetaclass(type):
         return cls in subcls.MRO
 
     def __instancecheck__(cls, instance):
+        if type(instance) is cls:
+            return True
+
         if getattr(type(instance), "__typed_python_category__", None) != "Class":
             return False
 
@@ -396,7 +428,7 @@ class FunctionOverloadArg:
 
 
 class FunctionOverload:
-    def __init__(self, functionTypeObject, index, code, funcGlobalsInCells, closureVarLookups, returnType):
+    def __init__(self, functionTypeObject, index, code, funcGlobalsInCells, closureVarLookups, returnType, signatureFunction, methodOf):
         """Initialize a FunctionOverload.
 
         Args:
@@ -410,6 +442,9 @@ class FunctionOverload:
                 actual function.
             returnType - the return type annotation, or None if None provided. (if None was
                 specified, that would be the NoneType)
+            signatureFunction - None, or the user-provided callable that determines the
+                return type as a function of the types of the arguments to the function.
+            methodOf - if we are a method of a class, the class object
         """
         self.functionTypeObject = functionTypeObject
         self.index = index
@@ -417,7 +452,9 @@ class FunctionOverload:
         self.functionCode = code
         self.funcGlobalsInCells = funcGlobalsInCells
         self.returnType = returnType
+        self.signatureFunction = signatureFunction
         self._realizedGlobals = None
+        self.methodOf = methodOf
         self.args = ()
 
     @property
@@ -442,6 +479,7 @@ class FunctionOverload:
             res = {}
 
             globalNames = set(self.extractGlobalNamesFromCode(self.functionCode))
+
             for name in self.functionGlobals:
                 if name.split(".")[0] in globalNames:
                     res[name] = self.functionGlobals[name]
@@ -449,6 +487,9 @@ class FunctionOverload:
             for varname, cell in self.funcGlobalsInCells.items():
                 res[varname] = cell.cell_contents
 
+            if self.methodOf is not None:
+                res['__class__'] = self.methodOf.Class
+
             self._realizedGlobals = res
 
         return self._realizedGlobals
@@ -475,6 +516,8 @@ class FunctionOverload:
         self.args = self.args + (FunctionOverloadArg(name, defaultVal, typeFilter, isStarArg, isKwarg),)
 
     def __str__(self):
+        if self.methodOf:
+            return "FunctionOverload(%s, returns %s, %s)" % (self.methodOf.Class.__name__, self.returnType, self.args)
         return "FunctionOverload(returns %s, %s)" % (self.returnType, self.args)
 
     def _installNativePointer(self, fp, returnType, argumentTypes):
